﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Tilemaps;
using UnityEngine.UI;

public class GenerateWorld : MonoBehaviour {

	public static GenerateWorld instance;

	public int worldHeight;
	public int worldWidth;
	public List<List<int>> Grid;

	private List<Tilemap> LandscapeLayers = new List<Tilemap>();
	private List<Tilemap> OverlayLayers = new List<Tilemap>();
	private List<Tilemap> BorderLayers = new List<Tilemap>();
	public Tilemap LandscapeTiles;
	public Tilemap OverlayTiles;
	public Tilemap BorderTiles;
	public TileBase tileWater;
	public TileBase tileLevee;
	public TileBase tileCity;
	public TileBase tileCrackSingle;
	public TileBase tileCrackMultiple;
	public TileBase tileBorder;

	private int LeveeHealth;
	public Text instruction;
	public Text shadow;
	private Image gameOverCard;
	private Image victoryCard;

	public Text gameOverText;
	public Text tryAgainText;
	public Text victoryText;
	public Text playAgainText;

	private AudioSource soundMaker;
    public AudioClip repairWall;

    private int[] Wave1 = new int[] { 5, 5, 2, 2, 2, 2, 1, 1, 1, 1 };
    private int[] Wave2 = new int[] { 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1 };
    private int[] Wave3 = new int[] { 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };

    public bool GameOver;
   
	void Awake() {
   		LandscapeLayers.Add(LandscapeTiles);
   		OverlayLayers.Add(OverlayTiles);

   		//generate levee stats matrix
   		LeveeHealth = 2;
		Grid = new List<List<int>>();
		for(int i = 0; i < 10; i++) {
			Grid.Add (new List<int>());
			for (int j = 0; j < 3; j++) {
				Grid[i].Add (LeveeHealth);
			}
		}

		soundMaker = GameObject.Find("Audio Source").GetComponent<AudioSource>();
		instruction = GameObject.Find("WaveText").GetComponent<Text>();
		shadow = GameObject.Find("Shadow").GetComponent<Text>();
		instruction.enabled = false;
		shadow.enabled = false;
		gameOverCard = GameObject.Find("GameOver").GetComponent<Image>();
		victoryCard = GameObject.Find("Victory").GetComponent<Image>();
		gameOverCard.enabled = false;
		victoryCard.enabled = false;

		gameOverText = GameObject.Find("GameOverText").GetComponent<Text>();
		tryAgainText = GameObject.Find("TryAgainText").GetComponent<Text>();
		victoryText = GameObject.Find("VictoryText").GetComponent<Text>();
		playAgainText = GameObject.Find("PlayAgainText").GetComponent<Text>();
		gameOverText.enabled = false;
		tryAgainText.enabled = false;
		victoryText.enabled = false;
		playAgainText.enabled = false;
	}

    // Start is called before the first frame update
    void Start() {

    	instance = this;

    	for(int i = 0; i < 10; i++) {
			for (int j = 0; j < 10; j++) {
				LandscapeLayers[0].SetTile(new Vector3Int(j, i, 0), tileWater);
			}
		}

		for(int i = 0; i < 10; i++) {
			for (int j = 10; j < 13; j++) {
				LandscapeLayers[0].SetTile(new Vector3Int(j, i, 0), tileLevee);
			}
		}

		StartCoroutine(SpawnSequence());
    }

    void Update(){
    	if (Input.GetMouseButtonDown(0) && GameOver == true){
    		//Time.timeScale = 1.0f;
			UnityEngine.SceneManagement.SceneManager.LoadScene("Levee1");
		}
    }

    IEnumerator SpawnSequence (){
    	if(GameOver == false){
	    	yield return new WaitForSeconds (2);
	    	instruction.enabled = true;
	    	shadow.enabled = true;
	    	yield return new WaitForSeconds (2);
	    	instruction.enabled = false;
	    	shadow.enabled = false;

	    	for (int i = 0; i < 10; i++){
		    	SpawnAliens();
		        yield return new WaitForSeconds (Wave1[i]);
		    }

		    yield return new WaitForSeconds (10);
		    if(GameOver == false){
			    instruction.text = "Second Wave Incoming!";
		    	instruction.enabled = true;
		    	shadow.text = "Second Wave Incoming!";
		    	shadow.enabled = true;
		    	yield return new WaitForSeconds (2);
		    	instruction.enabled = false;
		    	shadow.enabled = false;

			    for (int i = 0; i < 15; i++){
			    	SpawnAliens();
			        yield return new WaitForSeconds (Wave2[i]);
			    }
			}

		    yield return new WaitForSeconds (10);
		    if(GameOver == false){
			    instruction.text = "Final Wave Incoming!";
		    	instruction.enabled = true;
		    	shadow.text = "Final Wave Incoming!";
		    	shadow.enabled = true;
		    	yield return new WaitForSeconds (2);
		    	instruction.enabled = false;
		    	shadow.enabled = false;

			    for (int i = 0; i < 20; i++){
			    	SpawnAliens();
			        yield return new WaitForSeconds (Wave3[i]);
			    }
			}

		    yield return new WaitForSeconds (10);
		    if (GameOver == false){
				victoryText.enabled = true;
				playAgainText.enabled = true;
				victoryCard.enabled = true;
		    	GameOver = true;
		    }
		}
    }

    void SpawnAliens(){
    	ObjectPool.instance.GetObjectForType("Alien", false);
    }

    public void DamageLevee(int xCoordinate, int yCoordinate){
    	Grid[yCoordinate][xCoordinate] -= 1;
    	if (Grid[yCoordinate][xCoordinate] == 2){
    		OverlayLayers[0].SetTile(new Vector3Int(xCoordinate + 10, yCoordinate, 0), tileCrackSingle);
    	}
    	if (Grid[yCoordinate][xCoordinate] == 1){
    		OverlayLayers[0].SetTile(new Vector3Int(xCoordinate + 10, yCoordinate, 0), tileCrackMultiple);
    	}
    	if (Grid[yCoordinate][xCoordinate] <= 0){
    		LandscapeLayers[0].SetTile(new Vector3Int(xCoordinate + 10, yCoordinate, 0), tileWater);
    		for(int i = -1; i < 2; i++) {
				for (int j = -1; j < 2; j++) {
					if (Grid[yCoordinate][xCoordinate] > 0)
						LandscapeLayers[0].SetTile(new Vector3Int(xCoordinate + 10 + j, yCoordinate + i, 0), tileLevee);
						OverlayLayers[0].SetTile(new Vector3Int(xCoordinate + 10, yCoordinate, 0), null);
				}
			}
			if (xCoordinate == 2){
				for(int i = 0; i < 10; i++) {
					for (int j = 13; j < 20; j++) {
						LandscapeLayers[0].SetTile(new Vector3Int(j, i, 0), tileWater);
						GameOver = true;
						//Time.timeScale = 0.0f;
						StartCoroutine(Lose());
					}
				}
			}
    	}
    }

    IEnumerator Lose (){
    	yield return new WaitForSeconds (3);
    	gameOverText.enabled = true;
		tryAgainText.enabled = true;
	    gameOverCard.enabled = true;
    }

    public void RepairLevee(int xCoordinate, int yCoordinate){
     	if (Grid[yCoordinate][xCoordinate] > 0 & Grid[yCoordinate][xCoordinate] < LeveeHealth){
	       	Grid[yCoordinate][xCoordinate] += 1;
	       	soundMaker.PlayOneShot(repairWall);
	       	if (Grid[yCoordinate][xCoordinate] == LeveeHealth - 1)
	       		OverlayLayers[0].SetTile(new Vector3Int(xCoordinate + 10, yCoordinate, 0), tileCrackSingle);
	       	else if (Grid[yCoordinate][xCoordinate] == LeveeHealth)
	       		OverlayLayers[0].SetTile(new Vector3Int(xCoordinate + 10, yCoordinate, 0), null);
		}
    }

}
